import java.io.*;

// error in writing code

/**
 * RandomAccessFile with a buffer to improve performance
 *
 * Based on code from http://www.javaworld.com/javaworld/javatips/jw-javatip26.html
 *  and
 * http://java.sun.com/j2se/1.3/docs/api/java/io/
 *
 * Be sure to call flush() before closing to ensure that all data is written
 * (NOTE: WRITING NOT YET WORKING!!!)
 */
public class BufferedRandAccessFile extends RandomAccessFile {

        /* For efficiency reasons, we define a byte buffer instead of char buffer.
         * The variables buf_end, buf_pos, and real_pos are used to record the
         *  effective positions on the buffer:
         */
        private byte buffer[];

        private int buf_end = 0;
        private int buf_pos = 0;
        private int lastWriteLoc = 0; // the last byte written to disk
        private long real_pos = 0;
        private final int BUF_SIZE;

        // A new constructor is added with an additional parameter to specify the size of the buffer:


        public BufferedRandAccessFile(String filename, String mode, int bufsize)
         throws IOException {
                          super(filename,mode);
                          invalidate();
                          if (bufsize > 65536) // adjust writeUTFBuff() so this can be lowered
                             BUF_SIZE = bufsize;
                          else
                             BUF_SIZE = 65536;
                          buffer = new byte[BUF_SIZE];
        }

        /** The new read method is written such that it always reads from the buffer first. It overrides the
         *  native read method in the original class, which is never engaged until the buffer has run out of
         *  room. In that case, the fillBuffer method is called to fill in the buffer.
         *  In fillBuffer, the original read is
         * invoked. The private method invalidate is used to indicate that the buffer no longer
         * contains valid
         * contents. This is necessary when the seek method moves the file pointer out of the buffer.
         */
        public final int read() throws IOException {
                          if (buf_pos >= buf_end) {
                             if ( fillBuffer() < 0 )
                                return -1;
                          }
                          if (buf_end == 0) {
                               return -1;
                          } else {
                               return buffer[buf_pos++];
                          }
        }

        /**
         * Flushes (writes the contents of the buffer immediately)
         *  call automatically when buffer fills up
         */
        public void flush() throws IOException {

		System.out.println("flush called");

        	if ( buf_pos > 0 ) {
        	   // write the part of the buffer that is used to disk
        	   super.write(buffer, 0, buf_pos - 1);
                   System.out.println(buf_pos+" bytes written");
                   invalidate();

                }
                buf_end = BUF_SIZE - 1;

        }

	/**
	 * Reads a long 64-bit integer
	 */
        public final long readLongBuff() throws IOException {

		long l = 0L;

                if (buf_pos >= buf_end) {
                   if ( fillBuffer() < 0 )
                      throw new IOException();
                }

                if (buf_end == 0) {
                   throw new EOFException();
                }
                else {

                   l =     ( ( (long)(buffer[buf_pos++] & 0xff) << 56) |
                             ( (long)(buffer[buf_pos++] & 0xff) << 48) |
                             ( (long)(buffer[buf_pos++] & 0xff) << 40) |
                             ( (long)(buffer[buf_pos++] & 0xff) << 32) |
                             ( (long)(buffer[buf_pos++] & 0xff) << 24) |
                             ( (long)(buffer[buf_pos++] & 0xff) << 16) |
                             ( (long)(buffer[buf_pos++] & 0xff) <<  8) |
                             ( (long)(buffer[buf_pos++] & 0xff)      )
                           );

                }

                return l;

        }

        /**
	 * Reads a long 64-bit integer
	 */
        public final void writeLongBuff(long l) throws IOException {

                // check to see if the buffer has room for a long (8 bytes)
                if ( BUF_SIZE - buf_pos <= 8 ) {
                   flush(); // write the buffer to disk
                }

		// convert long to bytes and write to buffer
		buffer[buf_pos++] = (byte)( (int)(l >>> 56) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 48) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 40) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 32) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 24) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 16) & 0xff );
                buffer[buf_pos++] = (byte)( (int)(l >>> 8)  & 0xff );
                buffer[buf_pos++] = (byte)( (int)l          & 0xff );

	}

	/**
	 * Reads a unsigned 16-bit integer
	 */
	public final int readUnsignedShortBuff() throws IOException {

        	int i = read();
        	int j = read();
                if ( (i | j) < 0 )
                   throw new EOFException();
                else
                   return (i << 8) + j;

        }

        public final int readUnsignedByteBuff() throws IOException {

        	int i = read();
        	if (i < 0)
                   throw new EOFException();
        	else
                   return i;

        }

	/**
	 * Reads a UTF string
	 */
	public final String readUTFBuff() throws IOException {

                int utfLen = readUnsignedShortBuff(); // first two bytes are the UTF length

                char chBuff[] = new char[utfLen];

                int j = 0; // byte counter
                int numChars = 0; // number of characters

                while (j < utfLen) {
                      // bytes are then converted to characters by
                      // considering them in groups. The length of each group is computed
                      // from the value of the first byte of the group. The byte following
                      // a group, if any, is the first byte of the next group.
                      int byte1 = readUnsignedByteBuff(); // first byte of group
                      switch (byte1 >> 4) { // shift right so we're looking at the high 4 bits of the byte
            		     case 0: // '\0'
                             case 1: // '\001'
                             case 2: // '\002'
            		     case 3: // '\003'
                 	     case 4: // '\004'
            		     case 5: // '\005'
             		     case 6: // '\006'
            		     case 7: // '\007'
            		             // If the first byte of a group matches the bit pattern 0xxxxxxx
                                     // (where x means "may be 0 or 1") (ie 0-7 after the shift)
                                     // then the group consists of
                                     // just that byte. The byte is zero-extended to form a character.
                                     j++;
                                     chBuff[numChars++] = (char)byte1;
                                     break;
                             case 12: // '\f'
                             case 13: // '\r'
                                      // If the first byte of a group matches the bit pattern 110xxxxx
                                      // (ie 12 or 13 after the shift),
                                      // then the group consists of that byte a and a second byte b.
                                      // If there is no byte b (because byte a was the last of the bytes
                                      // to be read), or if byte b does not match the bit pattern 10xxxxxx,
                                      // then a UTFDataFormatException is thrown.
                                      if ( (j += 2) > utfLen )
                                         throw new UTFDataFormatException();
                                      int byte2 = readUnsignedByteBuff(); // the second byte
                                      if ( (byte2 & 0xc0) != 128)
                                         throw new UTFDataFormatException();
                                      chBuff[numChars++] = (char)((byte1 & 0x1f) << 6 | byte2 & 0x3f);
                                      break;
                             case 14: // '\016'
                                      // 1110XXXX
                                      if ( (j += 3) > utfLen )
                                         throw new UTFDataFormatException();
                                      int j1 = readUnsignedByteBuff();
                                      int k1 = readUnsignedByteBuff();
                                      if ( (j1 & 0xc0) != 128 || (k1 & 0xc0) != 128 )
                                         throw new UTFDataFormatException();
                                      chBuff[numChars++] = (char)((byte1 & 0xf) << 12 | (j1 & 0x3f) << 6 | k1 & 0x3f);
                                      break;

                             case 8:  // '\b'
                             case 9:  // '\t'
                             case 10: // '\n'
                             case 11: // '\013'
                             default:
                                      throw new UTFDataFormatException();
                      } // end switch
                } // end while loop
                return new String(chBuff, 0, numChars);

	}

	/**
	 * Writes a UTF string
	 */
	public final void writeUTFBuff(String s) throws IOException {

		int numChars = s.length(); // length of string in terms of characters
        	int numBytes = 0; // length of string in bytes (UTF format)

        	// compute numBytes
        	for (int k = 0; k < numChars; k++) {
        	    char c = s.charAt(k);
            	    if (c >= '\001' && c <= '\177')
                       numBytes++;
                    else if (c > '\u07FF')
                       numBytes += 3;
                    else
                       numBytes += 2;
                }

        	if (numBytes > 65535)
                   throw new UTFDataFormatException();

                // check to see if the buffer has room for the string (numBytes bytes)
                if ( BUF_SIZE - buf_pos <= numBytes ) {
                   flush(); // write the buffer to disk
                }

                buffer[buf_pos++] = (byte)(numBytes >>> 8 & 0xff);
                buffer[buf_pos++] = (byte)(numBytes & 0xff);

        	for (int l = 0; l < numChars; l++) {
                    char c1 = s.charAt(l);
                    if (c1 >= '\001' && c1 <= '\177') {
                       // If a character c is in the range \u0001 through \u007f, it is represented
                       // by one byte
                       buffer[buf_pos++] = (byte)c1;
                    }
                    else if (c1 > '\u07FF') {
                       // If a character c is in the range \u0800 through uffff, then it is represented
                       // by three bytes
                       buffer[buf_pos++] = (byte)(0xe0 | c1 >> 12 & 0xf);
                       buffer[buf_pos++] = (byte)(0x80 | c1 >> 6 & 0x3f);
                       buffer[buf_pos++] = (byte)(0x80 | c1 & 0x3f);
                    }
                    else {
                       // If a character c is \u0000 or is in the range \u0080 through \u07ff,
                       // then it is represented by two bytes
                       buffer[buf_pos++] = (byte)(0xc0 | c1 >> 6 & 0x1f);
                       buffer[buf_pos++] = (byte)(0x80 | c1 & 0x3f);
                    }
                } // end for loop

        }

        private int fillBuffer() throws IOException {
                          int n = super.read(buffer, 0, BUF_SIZE);
                          if (n >= 0) {
                            real_pos +=n;
                            buf_end = n;
                            buf_pos = 0;
                          }
                          return n;
        }

        private void invalidate() throws IOException {
                          buf_end = 0;
                          buf_pos = 0;
                          real_pos = super.getFilePointer();
        }

        /** The other parameterized read method also is overridden.
         *  The code for the new read is listed below.
         *  If there is enough buffer, it will simply call
         *  System.arraycopy to copy a portion of the buffer directly
         *  into the user-provided area. This presents the most
         *  significant performance gain because the read
         *  method is heavily used in the getNextLine method,
         *  which is our replacement for readLine.
         */
        public int read(byte b[], int off, int len) throws IOException {
                         int leftover = buf_end - buf_pos;
                         if (len <= leftover) {
                                   System.arraycopy(buffer, buf_pos, b, off, len);
                              buf_pos += len;
                              return len;
                         }
                         for(int i = 0; i < len; i++) {
                            int c = this.read();
                            if (c != -1)
                               b[off+i] = (byte)c;
                            else {
                               return i;
                            }
                         }
                         return len;
        }

        // The original methods getFilePointer and seek need to be overridden
        // as well in order to take advantage
        // of the buffer. Most of time, both methods will simply operate inside the buffer.

        public long getFilePointer() throws IOException{
                          long l = real_pos;
                          return (l - buf_end + buf_pos) ;
        }

        public void seek(long pos) throws IOException {
                          int n = (int)(real_pos - pos);
                          if (n >= 0 && n <= buf_end) {
                            buf_pos = buf_end - n;
                          } else {
                            super.seek(pos);
                            invalidate();
                          }
        }

        /* Most important, a new method, getNextLine, is added to replace the readLine method. We can not
         *  simply override the readLine method because it is defined as final in the original class.
         *  The getNextLine
         *  method first decides if the buffer still contains unread contents. If it doesn't,
         *  the buffer needs to
         *  be filled up. If the new line delimiter can be found in the buffer, then a new line
         *  is read from the
         *  buffer and converted into String. Otherwise, it will simply call the read method to
         *  read byte by byte.
         *  Although the code of the latter portion is similar to the original readLine,
         *  performance is better here
         *  because the read method is buffered in the new class.
         *
         * return a next line in String
         *
        public final String getNextLine() throws IOException {
                         String str = null;
                         if (buf_end-buf_pos <= 0) {
                            if (fillBuffer() < 0) {
                                      throw new IOException("error in filling buffer!");
                            }
                         }
                         int lineend = -1;
                         for(int i = buf_pos; i < buf_end; i++) {
                              if (buffer[i] == '\n') {
                               lineend = i;
                                break;
                                }
                         }
                         if (lineend < 0) {
                              StringBuffer input = new StringBuffer(256);
                              int c;
                                   while (((c = read()) != -1) && (c != '\n')) {
                                       input.append((char)c);
                              }
                              if ((c == -1) && (input.length() == 0)) {
                                return null;
                              }
                              return input.toString();
                         }
                         if (lineend > 0 && buffer[lineend-1] == '\r')
                              str = new String(buffer, 0, buf_pos, lineend - buf_pos -1);
                         else str = new String(buffer, 0, buf_pos, lineend - buf_pos);
                         buf_pos = lineend +1;
                         return str;
        }*/

}
