import java.awt.*;
import javax.swing.*;

import java.awt.dnd.*;
import java.awt.datatransfer.*;

import java.util.Hashtable;
import java.util.List;
import java.util.Iterator;

import java.io.*;
import java.io.IOException;

public class DNDDesktopPane extends JDesktopPane
                            implements DNDComponentInterface, DropTargetListener,
                                        DragSourceListener, DragGestureListener   {

         boolean dropable = true;
  	 boolean dragable = true;

  	 boolean canAdd = true;
  	 boolean canRemove = true;

  	 private ImageViewer imageViewer;

         /**
          * enables this component to be a dropTarget
          */

  	 DropTarget dropTarget = null;

  	 /**
   	  * enables this component to be a Drag Source
   	  */
        DragSource dragSource = null;

	public DNDDesktopPane(ImageViewer iv) {

		super();
		imageViewer = iv;
		dropTarget = new DropTarget (this, this);
    		dragSource = new DragSource();
    		dragSource.createDefaultDragGestureRecognizer( this, DnDConstants.ACTION_MOVE, this);

	}

	 public void setDropable(boolean b) { dropable = b; }

  public void setDragable(boolean b) { dragable = b; }

  public void setCanAdd(boolean b) { canAdd = b; }

  public void setCanRemove(boolean b) { canRemove = b; }

  /**
   * is invoked when you are dragging over the DropSite
   *
   */

  public void dragEnter (DropTargetDragEvent event) {

    event.acceptDrag (DnDConstants.ACTION_MOVE);
  }

  /**
   * is invoked when you are exit the DropSite without dropping
   *
   */

  public void dragExit (DropTargetEvent event) {
  }

  /**
   * is invoked when a drag operation is going on
   *
   */

  public void dragOver (DropTargetDragEvent event) {
  }

  /**
   * a drop has occurred
   *
   */


  public void drop (DropTargetDropEvent event) {

    try {
        Transferable transferable = event.getTransferable();

        // we accept only Strings
        if (transferable.isDataFlavorSupported (DataFlavor.stringFlavor)){

            event.acceptDrop(DnDConstants.ACTION_MOVE);
            String s = (String)transferable.getTransferData ( DataFlavor.stringFlavor);
            if (dropable)
               addElement( s );
            event.getDropTargetContext().dropComplete(true);
        }
        else{
            event.rejectDrop();
        }
    }
    catch (IOException exception) {
        exception.printStackTrace();
        System.err.println( "Exception" + exception.getMessage());
        event.rejectDrop();
    }
    catch (UnsupportedFlavorException ufException ) {
      ufException.printStackTrace();
      System.err.println( "Exception" + ufException.getMessage());
      event.rejectDrop();
    }
  }

  /**
   * is invoked if the use modifies the current drop gesture
   *
   */


  public void dropActionChanged ( DropTargetDragEvent event ) {
  }

  /**
   * a drag gesture has been initiated
   *
   */

  public void dragGestureRecognized( DragGestureEvent event) {

	// not implemented

  }

  /**
   * this message goes to DragSourceListener, informing it that the dragging
   * has ended
   *
   */

  public void dragDropEnd (DragSourceDropEvent event) {

    if ( event.getDropSuccess() ){
        if (dragable)
           removeElement();
    }

  }

  /**
   * this message goes to DragSourceListener, informing it that the dragging
   * has entered the DropSite
   *
   */

  public void dragEnter (DragSourceDragEvent event) {
  }

  /**
   * this message goes to DragSourceListener, informing it that the dragging
   * has exited the DropSite
   *
   */

  public void dragExit (DragSourceEvent event) {
  }

  /**
   * this message goes to DragSourceListener, informing it that the dragging is currently
   * ocurring over the DropSite
   *
   */

  public void dragOver (DragSourceDragEvent event) {
  }

  /**
   * is invoked when the user changes the dropAction
   *
   */

  public void dropActionChanged ( DragSourceDragEvent event) {
  }

  /**
   * adds elements to itself
   *
   */
  public void addElement(Object s) {

  	if (canAdd) {
  	   String dir = imageViewer.getCurrDir();  // get the current directory
  	   File file = new File( dir, s.toString() );
           imageViewer.showImageAI(file); // open the file
        }

  }

  public void removeElement() {

  	if (canRemove)
  	   return; // remove element (not implemented)

  }

}
