import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;

/**
 * View Full-Screen and Slide Show
 */
public class FullScreenWindow extends JWindow
                       implements ActionListener {

	private ImageInternalFrame selectedFrame;
        private JPanel contentPane;
        private JScrollPane scrollPane;

	private javax.swing.Timer timer;
	private Vector files;
	private Vector filenames;
	private int ct;
	private int timeDelay;
	private String dirName;
	private ViewPopupMenu popup;

	private boolean custom;

	/**
	 * View full screen
	 */
	public FullScreenWindow(ImageInternalFrame frame, ViewPopupMenu popup) {

		super();
		selectedFrame = frame;
		this.popup = popup;
		init();

	}

	private void init() {

                Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setSize( (int)d.getWidth(), (int)d.getHeight() );

                String filename = selectedFrame.getFilename();
                loadImage( new File(filename), true );

	}

	public void setVisible(boolean visible) {

		super.setVisible(visible);
		if (visible)
		   requestFocus();

	}

	/**
	 * Constructor for "playing" custom slide show
	 */
	public FullScreenWindow(int timeDelay, Vector filenames) {

		super();
		ct = 0;
		this.timeDelay = timeDelay;
		this.filenames = filenames;
		custom = true;
		setup();
		timer = new javax.swing.Timer(timeDelay, this);
		timer.start();

	}

	/**
	 * Starts a slide show for the files in
	 * the directory _dirName with a time delay
	 * (in milliseconds) of _timeDelay.
	 */
	public FullScreenWindow(int _timeDelay, String _dirName) {

		super();
		ct = 0;
		dirName = _dirName;
		timeDelay = _timeDelay;
		custom = false;
		setup();
		timer = new javax.swing.Timer(timeDelay, this);
		timer.start();
		files = Utils.getFiles(dirName);

	}

	/**
	 * Starts a slide show for the files in
	 * the directory _dirName.
	 */
	public FullScreenWindow(String _dirName) {

		super();
		ct = 0;
		dirName = _dirName;
		custom = false;
		setup();
		timeDelay = 2000;
		timer = new javax.swing.Timer(timeDelay, this);
		timer.start();
		files = Utils.getFiles(dirName);

	}

	private void setup() {

		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setSize( (int)d.getWidth(), (int)d.getHeight() );

	}

	public int getTimeDelay() { return timeDelay; }

	public void setTimeDelay(int _timeDelay) { timeDelay = _timeDelay; }

	public void actionPerformed(ActionEvent e) {

		File file;

        	if (custom && (filenames.size() > ct) ) {
		   file = new File( (String)( filenames.get(ct) ) );
        	   loadImage(file, false);
		   ct++;
        	}
        	else if ( (!custom) && (files.size() > ct) ) {
		   file = new File( dirName + files.get(ct) );
		   loadImage(file, false);
		   ct++;
		}
		else {
		   timer.stop();
		   setVisible(false);
		   dispose();
		}

	}

	private void loadImage(File file, boolean add) {

		// load the image
                PlanarImage planarImage = JAI.create("fileload", file.getAbsolutePath() );
                BufferedImage bufferedImage = planarImage.getAsBufferedImage();

        	// add the image to the scroll pane
        	DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	scrollPane = new JScrollPane(imagePanel);

        	if (add) {
        	   scrollPane.addMouseListener(popup);
                   scrollPane.addMouseMotionListener(popup);
                   addKeyListener(popup);
                   popup.enableFullScreenMode(this, selectedFrame);
                }

                scrollPane.setToolTipText("Click to close");

        	setContentPane(scrollPane);
        	validate();

	}

}

