import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import java.awt.*;

class ImageLoaderThread extends Thread {

	private static final String FILELOAD = "fileload";

	private ImageViewer iv;
	private File file;
	private File[] files;
	private Vector fileVector;
	private boolean multiFiles;
	private int ct, numFiles;

	public ImageLoaderThread(ImageViewer parent, Vector _files) {

		super();
		iv = parent;
		fileVector = _files;
		ct = 0;
		numFiles = fileVector.size();
		multiFiles = true;

	}

	public ImageLoaderThread(ImageViewer parent, File[] _files) {

		super();
		iv = parent;
		files = _files;
		ct = 0;
		numFiles = files.length;
		multiFiles = true;

	}

	public ImageLoaderThread(ImageViewer parent, File _file) {

		super();
		iv = parent;
		file = _file;
		multiFiles = false;

	}

	public void run() {

		if (multiFiles)
		   openMultiFiles();
		else
		   openFile(file);


        }

        private void openFile(File file) {

		String filename = file.getAbsolutePath();

		iv.setStatus( "Loading " + filename + "..." );

        	// load the image
                BufferedImage bufferedImage = null;
                MediaTracker tracker = new MediaTracker(iv); // used to fix command-line open bug on Solaris
                PlanarImage planarImage = JAI.create(FILELOAD, filename);
                try {
                     bufferedImage = planarImage.getAsBufferedImage();
                     //RenderedImage image = JAI.create(FILELOAD, filename);
                     //bufferedImage = (BufferedImage)image;
                }
                catch (Exception e) {
                     iv.setStatus("Unable to open " + filename);
                     return;
                }

                tracker.addImage(bufferedImage, 0);
                try {
                     tracker.waitForID(0);
                }
                catch (InterruptedException e) {
                     return;
                }

		if ( ( tracker.statusAll(false) & MediaTracker.ERRORED ) != 0)
                   return;

        	// add the image to the scroll pane
        	DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	tracker.removeImage(bufferedImage, 0);
        	JScrollPane scrollPane = new JScrollPane(imagePanel);

		// now runs on GUI thread
		//ImageInternalFrame frame = iv.addToFrame(scrollPane, file);
                //frame.setImage(bufferedImage); // set the frame's image

                int width = bufferedImage.getWidth() + 40;
                int height = bufferedImage.getHeight() + 40;

                doOpen(file, scrollPane, bufferedImage, width, height);

        }

        private void openMultiFiles() {

        	if (fileVector != null) {

        	   for (int i = 0; i < fileVector.size(); i++) {
                       ct++;
                       openFile( (File)( fileVector.get(i) ) );
                   }

        	}
        	else {

        	   for (int i = 0; i < files.length; i++) {
                       ct++;
                       openFile(files[i]);
                   }

                }

        }

        private void doOpen(File _file, JScrollPane _scrollPane, BufferedImage _bufferedImage,
                             int _width, int _height) {

		final ImageInternalFrame frame;
		final File file = _file;
		final JScrollPane scrollPane = _scrollPane;
		final BufferedImage bufferedImage = _bufferedImage;
		final int width = _width;
		final int height = _height;

		// Runs on the event-dispatching thread
		Runnable open = new Runnable() {
          	   public void run() {
          	       ImageInternalFrame frame = iv.addToFrame(scrollPane, file);
                       frame.setImage(bufferedImage); // set the frame's image
                       iv.openFrame(frame, width, height);
                       iv.setDefaultStatus();
                       if (multiFiles) {
                          int progress = ct * 100;
                          iv.setProgress( progress/numFiles );
                       }
                   }
      		};
                SwingUtilities.invokeLater(open);

        }

}
