import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.*;
import javax.swing.*;
import java.io.*;

class ImageWriter {

	private static JPEGEncodeParam jpegEncodeParam;
	private static BMPEncodeParam bmpEncodeParam;
	private static PNGEncodeParam pngEncodeParam;
	private static TIFFEncodeParam tiffEncodeParam;

	/**
	 * format - format of the image (JPEG, BMP, PNG, or TIFF)
	 * outputFilename is the file to save to
	 * src is the image to save
	 * quality is the image quality (between 0.0 and 1.0), with 1 being the best quality,
	 *  -1 gives the default quality
	 */
	public static void save(String format, String outputFilename, RenderedImage src, float quality) {

		ImageEncoder enc = null;

		OutputStream os = null;

		try {
		     os = new FileOutputStream(outputFilename);
                }
                catch (FileNotFoundException fnfe) {
                      ImageViewer.setStatusStatic("Unable to save "+outputFilename);
                      return;
                }
                try {
		     if ( format.equals("JPEG") ) {
		        if (jpegEncodeParam == null)
		           jpegEncodeParam = new JPEGEncodeParam();
                        if (quality != -1)
                           jpegEncodeParam.setQuality(quality);
                        else
                           jpegEncodeParam.setQuality(0.75F);
                        enc = ImageCodec.createImageEncoder(format, os, jpegEncodeParam);
                     }
                     else if ( format.equals("BMP") ) {
                         if (bmpEncodeParam == null)
                            bmpEncodeParam = new BMPEncodeParam();
                         enc = ImageCodec.createImageEncoder(format, os, bmpEncodeParam);
                     }
                     else if ( format.equals("PNG") ) {
                         if (pngEncodeParam == null)
                            pngEncodeParam = PNGEncodeParam.getDefaultEncodeParam(src);
                         enc = ImageCodec.createImageEncoder(format, os, pngEncodeParam);
                     }
                     else if ( format.equals("TIFF") ) {
                         if (tiffEncodeParam == null)
                            tiffEncodeParam = new TIFFEncodeParam();
                         enc = ImageCodec.createImageEncoder(format, os, tiffEncodeParam);
                     }
                }
                catch (Exception e) {
                      ImageViewer.setStatusStatic("Unable to save "+outputFilename);
                      return;
                }

                try {
                     enc.encode(src);
                     os.close();
                }
                catch (IOException ioe) {
                      ImageViewer.setStatusStatic("Unable to save "+outputFilename);
                      return;
                }
                ImageViewer.setStatusStatic(outputFilename + " saved");

	}

	public static void save(String format, String outputFilename, RenderedImage src) {

		save(format, outputFilename, src, -1);

	}


}
