import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.io.*;

/**
 * The "Save As" dialog.
 */
class SaveDialog {

	private ImageViewer parent;

	private JFileChooser chooser;
	private JSlider slider;


	/**
	 * dir is the directory to open the dialog
	 */
	public SaveDialog(ImageViewer parent, String dir, boolean useFileFilters,
	                   boolean useFileTypeIcons, boolean previewImages) {

		this.parent = parent;

		buildGUI(dir, useFileFilters, useFileTypeIcons, previewImages);

	}

	private void buildGUI(String dir, boolean useFileFilters, boolean useFileTypeIcons,
	                       boolean previewImages) {

		// set a vertical slider and maybe an image preview as the filechooser's accessory

		Box accessory = new Box(BoxLayout.Y_AXIS);

                accessory.add( createSlider() );

		chooser = new JFileChooser(dir);
		if (useFileFilters) {
		   chooser.addChoosableFileFilter( new JPEGFilter() );
                   chooser.addChoosableFileFilter( new PNGFilter() );
                   chooser.addChoosableFileFilter( new TIFFFilter() );
                   chooser.addChoosableFileFilter( new BMPFilter() );
                   chooser.setFileFilter( new ImageFilter() );
                }
                if (useFileTypeIcons)
                   chooser.setFileView( new ImageFileView() );
                if (previewImages) {
                   ImagePreview ip = new ImagePreview(chooser, parent);
                   accessory.add( Box.createVerticalStrut(20) );
                   accessory.add(ip);
                }

		JPanel jComp = new JPanel();
		jComp.add(accessory);
                chooser.setAccessory(jComp);

                chooser.setPreferredSize( new Dimension(600, 400) );

	}

	private JPanel createSlider() {

		slider = new JSlider(JSlider.HORIZONTAL, 0, 100, 75);
                slider.setMajorTickSpacing(25);
                slider.setMinorTickSpacing(5);
                slider.setPaintTicks(true);
                slider.setPaintLabels(true);
                slider.setSnapToTicks(true);
		slider.setToolTipText("Image Quality");

		JPanel q = new JPanel();
		q.add( new JLabel("Quality: ") );
		q.add(slider);

		return q;

	}

	/**
	 * Opens the "Save As" dialog
	 */
	public void show() {

		int returnVal = chooser.showSaveDialog(parent); // open the file chooser

                if (returnVal == JFileChooser.APPROVE_OPTION)
                   saveImage();

	}

	private void saveImage() {

		// get filename to save as
		File file = chooser.getSelectedFile();
		String ext = Utils.getExtension(file);

		// get the selected internal frame
		ImageInternalFrame iif = parent.getSelectedFrame();
                // get the image for that frame from its copy of DisplayJAI
                JScrollPane sp = (JScrollPane)iif.getContentPane();
		JViewport vp = sp.getViewport();
		DisplayJAI d = (DisplayJAI)vp.getView();
		RenderedImage img = d.getImage();

		// save the image
		String format;
		if ( ext.equals(Utils.jpg) || ext.equals(Utils.jpeg) )
		   format = "JPEG";
		else if ( ext.equals(Utils.png) )
		   format = "PNG";
		else if ( ext.equals(Utils.bmp) )
		   format = "BMP";
		else if ( ext.equals(Utils.tif) || ext.equals(Utils.tiff) )
		   format = "TIFF";
		else {
		   JOptionPane.showMessageDialog(parent, "Please select a valid image format" +
		                                  "(jpg, png, bmp, or tif",
                                                   "Error", JOptionPane.ERROR_MESSAGE);
                   return;
		}

		float quality = (float)( slider.getValue() )/(float)100;
		SaveThread st = new SaveThread(format, file, img, quality);
		st.start();

	}

	public void setCurrentDirectory(File dir) {

		chooser.setCurrentDirectory(dir);

	}

	public File getCurrentDirectory() {

		return chooser.getCurrentDirectory();

	}

}

