import java.awt.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.JPEGEncodeParam;
import javax.swing.*;
import java.io.*;
import java.util.*;

class Scale {

	private ImageViewer parent;
        private JPEGEncodeParam encodeParam;
        private float encodeQuality = 0.5F;

        //private static Properties properties;
	private static String userHomeS;
	private static String fileWPath;
	private static Hashtable hashTable; // the "fail list"
	private static boolean propsLoaded = false;
	private static BufferedRandAccessFile braf; // for reading
	private static RandomAccessFile raf; // for writing

	public Scale(ImageViewer parent, boolean useEncodeParam) {

		this.parent = parent;

		// set up encode param for thumbnails
		if (useEncodeParam) {
        	   encodeParam = new JPEGEncodeParam();
                   encodeParam.setQuality(encodeQuality);
                }

		userHomeS = System.getProperty(IVConstants.userHomeS) + ImageViewer.slashS; // user home

                if (!propsLoaded)
                   loadProperties();

	}

	public float getEncodeQuality() {
		return encodeQuality;
	}

	public void setEncodeQuality(float qual) {
		encodeQuality = qual;
	}

	/**
	 * Creates a JPEG thumbnail from inputFile and
	 * saves it to disk in outputFile. scaleWidth
	 * is the width to scale the image to
	 */
	public String createThumbnail(File inputFile, File outputFile, int scaleWidth) {

          String outputFilename = null;

          try {
	   File parDir = outputFile.getParentFile();

	   File parFile = inputFile.getParentFile();

           if ( !parDir.exists() )
              parDir.mkdir(); // create directory for thumbnails
           outputFilename = outputFile.getAbsolutePath();

           if ( hasFailed(inputFile) )
              return null;

           // Load the input image.
           RenderedOp src = JAI.create( "fileload", inputFile.getAbsolutePath() );
           BufferedImage bufferedImage = src.getAsBufferedImage();  // crashes here

           // resize the image so it has a width of scaleWidth
           //  (if it's current width is > scaleWidth)
           float width = bufferedImage.getWidth();
           float scale = 1/(width/(float)scaleWidth);
           if (scale < 1)
              src = scaleActual(scale, scale, bufferedImage, Interpolation.INTERP_NEAREST);
                                     // nearest interpolation is faster for scaling images

           // Encode the file as a JPEG image.
           FileOutputStream stream = new FileOutputStream(outputFilename);
           JAI.create("encode", src, stream, "JPEG", encodeParam);

           // Store the image in the JPEG format.
           JAI.create("filestore", src, outputFilename, "JPEG", encodeParam);


          }
          catch (Exception e) {
           if ( (inputFile != null) && ( inputFile.getName() != "null" ) )
              addElemToFailList(inputFile);
          }

          return outputFilename;

	}

	/**
	 * Checks to see if the creation of a particular thumbnail
	 * has previously failed
	 */
	private boolean hasFailed(File file) {

		String filename = file.getAbsolutePath();

		String lastModS; // the date the file was last modified
		lastModS = (String)hashTable.get(filename);

		if (lastModS == null)
		   return false;
		else {
                   // check the modified date to see if the file has
                   //  been modified since the failure

		   return true;
		}

	}

	public void addElemToFailList(File file) {

		String filename = file.getAbsolutePath();
		if ( filename.endsWith("null") )
		   return;
		long lastModified = file.lastModified(); // the date the file was last modified
		Long lastMod = new Long(lastModified);
		hashTable.put(filename, lastMod); // key, value

		saveFailList(filename, lastModified);

	}

	/**
	 * Saves a fail list element (ie a file/time pair)
	 */
	private void saveFailList(String s, long l) {

		/*properties.setProperty( IVConstants.THUMB_FAIL_LIST, hashTable.toString() );

		try {
                     FileOutputStream out =
                      new FileOutputStream(fileWPath);
                     BufferedOutputStream bos = new BufferedOutputStream(out);
                     properties.store(bos, IVConstants.THUMB_FAIL_LIST);
                     bos.close();
                     out.close();
                }
                catch (IOException ioe) {
                     System.err.println("thumbnail failure list could not be saved");
                     ioe.printStackTrace();
                }*/

                try {
                    if (raf == null) {
                       fileWPath = userHomeS + IVConstants.THUMB_FAIL_FILENAME;
                       raf = new RandomAccessFile(fileWPath, "rw");
                    }
                    raf.writeUTF(s);
                    raf.writeLong(l);
                }
                catch (IOException ioe) {
                    System.err.println("thumbnail failure list could not be saved");
                    ioe.printStackTrace();
                }

	}

	/**
         * Opens up the fail list file.
         */
	private void loadProperties() {

     		/*properties = new Properties();
     		FileInputStream in = null;
     		BufferedInputStream bis = null; */
     		fileWPath = userHomeS + IVConstants.THUMB_FAIL_FILENAME;
     		try {
     		     raf = new RandomAccessFile(fileWPath, "rw");
     		     braf = new BufferedRandAccessFile(fileWPath, "rw", 1000);
     		     /*in = new FileInputStream(fileWPath);
                     bis = new BufferedInputStream(in);
                     properties.load(bis);
	             String failListString = properties.getProperty(IVConstants.THUMB_FAIL_LIST);
	             */
	             parseFailList(/*failListString*/);
	        }
                catch (Exception e) {
                     System.out.println("fail file does not exist");
                     hashTable = new Hashtable();
                     raf = null;
                     braf = null;
                }
                finally {
                     propsLoaded = true;
                }

        }

        private void parseFailList(/*String list*/) {

                hashTable = new Hashtable();
                /*
		// get rid of leading "{" and trailing "}"
                list = list.substring( 1, list.length() - 1 );

                StringTokenizer st = new StringTokenizer(list, ",=");
                */
                String filename;
		long len, bytesRead;

                len = 0;

		try {
		    len = raf.length();
		}
		catch (Exception e1) {
		    return;
		}

		bytesRead = 0;
                while (bytesRead < len) {
                   try {
                      filename = braf.readUTFBuff();
                      int sLen = filename.length();
                      bytesRead += sLen;
                      filename = filename.trim();
                      /*if ( st.hasMoreTokens() ) {*/
                         /*lastModS = st.nextToken();*/
                         Long lastMod = new Long( braf.readLongBuff() );
                         bytesRead += 8;
                         hashTable.put(filename, lastMod); // key, value
                      /*}*/
                   }
                   catch (Exception e) {
                      // an exception will be thrown, since there is no way
                      //  to accurately determine the length in bytes of each string
                      //e.printStackTrace();
                      break;
                   }
                }
                // seek to braf's current position
                try {
                     raf.seek( braf.getFilePointer() );
                }
                catch (IOException ioe) {}
        }

	/**
         * zoom is equal to IVConstants.SCALE if this method was called on a scale,
         *  IVConstants.ZOOM_IN if called on a zoom in, IVConstants.ZOOM_OUT if called on a zoom out,
         *   or IVConstants.ZOOM_11 if zooming 1:1
         */
	public BufferedImage scale(String scaleString, ImageInternalFrame selectedFrame, int zoom) {

		BufferedImage bufferedImage = null;
		String backupSS = scaleString;

		if ( scaleString.endsWith(IVConstants.xS) ) // remove the "x" from the end
                   scaleString = scaleString.substring(0, scaleString.length() - 1);

                int index = scaleString.indexOf('/'); // see if the selected item is a fraction

                float scale;
                try {

                    if (index == -1) // integer
                       scale = Float.parseFloat(scaleString);
                    else { // fraction (ie 1/2)
                       float n, d;
                       n = Float.parseFloat( scaleString.substring(0,1) ); // the numerator
                       d = Float.parseFloat( scaleString.substring(2) ); // the denominator
                       scale = n/d;
                    }

                    if (zoom == IVConstants.ZOOM_IN) {
                       scale *= 2F; // zoom in (2x)
                       backupSS = Float.toString(scale) + IVConstants.xS;
                    }
                    else if (zoom == IVConstants.ZOOM_OUT) {
                       scale *= .5F; // zoom out (.5x)
                       backupSS = Float.toString(scale) + IVConstants.xS;
                    }
                    else if (zoom == IVConstants.ZOOM_11) {
                       scale = 1F; // zoom 1:1
                       backupSS = Float.toString(scale) + IVConstants.xS;
                    }

		    bufferedImage = scaleImage(scale, selectedFrame);

		    selectedFrame.setScaleFactor(backupSS);
                    parent.getScaleChoice().getEditor().setItem(backupSS);

                }
                catch (NumberFormatException nfe) {
                      JOptionPane.showMessageDialog(parent, "\""+scaleString+"\" is not a valid scale factor.",
		                                     "Error", JOptionPane.ERROR_MESSAGE);
                }

                return bufferedImage;

	}

	private BufferedImage scaleImage(float scale, ImageInternalFrame selectedFrame) {

		parent.setStatus("Scaling/Zooming...");

                BufferedImage bufferedImage = (BufferedImage)selectedFrame.getImage();
                RenderedOp rop = scaleActual(scale, scale, bufferedImage, parent.getScaleType() );

                // convert rop to a BufferedImage to improve performance
		bufferedImage = rop.getAsBufferedImage();

                return bufferedImage;

        }

        /**
         * Performs the actual image scaling.
         * interpType is the type of interpolation
         *  (ie bilinear, nearest from the Interpolation class)
         *  to use
         */
        private RenderedOp scaleActual(float scaleX, float scaleY,
                                        BufferedImage bufferedImage,
                                         int interpType) {

		// Create a interpolation object to be
                //  used with the "scale" operator.
                Interpolation interp = Interpolation.getInstance(interpType);

		// Stores the required input source and parameters in a
                //  ParameterBlock to be sent to the operation registry,
                //   and eventually to the "scale" operator.
                ParameterBlock pb = new ParameterBlock();
                pb.addSource(bufferedImage);
                pb.add(scaleX);
                pb.add(scaleY);
                pb.add(0.0F);
                pb.add(0.0F);
                pb.add(interp);

		// Create an operator to scale the image
                RenderedOp image2 = JAI.create(IVConstants.scaleS, pb);

		return image2;

        }

        public void showScaledImage(BufferedImage bufferedImage, ImageInternalFrame selectedFrame) {

		JScrollPane sp = (JScrollPane)selectedFrame.getContentPane();
		JViewport vp = sp.getViewport();
		DisplayJAI d = (DisplayJAI)vp.getView();
		d.setImage(bufferedImage);

		int width = bufferedImage.getWidth();
		int height = bufferedImage.getHeight();
		vp.setViewSize( new Dimension(width,height) ); // resize the scrollpane's viewport
		sp.revalidate();

		boolean tf = parent.getProp(PropertiesConstants.resizeOnScaleS);
		if (tf) { // resize the internal frame to better fit the image
		   width += 40;
		   height += 40;
		   selectedFrame.setIFrameSize(width, height, selectedFrame);
		}
		parent.setDefaultStatus();

        }


}
