import java.io.*;
import java.util.*;

class ThumbnailThread extends Thread {

	private ImageViewer imageViewer;
	private Queue queue;
	private boolean threadSuspended;

	/**
	 * True if one or less ThumbnailThreads are running.
	 */
	/*private static boolean canRun = true;*/

	private static Scale scale;

	public ThumbnailThread(Vector _fileNames, File _parentDir, ImageViewer _imageViewer) {

		threadSuspended = false;
		imageViewer = _imageViewer;
		scale = new Scale(imageViewer, true);
		queue = new Queue();
		ThumbDirInfo tdi = new ThumbDirInfo(_fileNames, _parentDir);
		queue.add(tdi);
                setPriority(Thread.MIN_PRIORITY);

	}

	/*private synchronized void canIRun() {

		while (!canRun) {

		   try {
                        wait();
                   }
                   catch (InterruptedException e) {}

                }
                canRun = false;

	}*/

	public void run() {

		ThumbDirInfo tdi;

                /*canIRun();*/
                while (true) {

                      if ( !( queue.isEmpty() ) ) {
                         tdi = (ThumbDirInfo)queue.getCurr();
                         createThumbs( tdi.getFileNames(), tdi.getParentDir() );
                      }
                      else { // suspend thread
                         try {
                              synchronized(this) {
                                  while (threadSuspended)
                                        wait();
                              }
                         }
                         catch (InterruptedException e) {}
                      }

                }

		/*letRun();*/

	}

	public /*synchronized*/ void add(Vector _fileNames, File _parentDir) {
		// should be synchronized (ie producer/consumer) including Queue ??
		//  seems to execute in order w/o sync though

		ThumbDirInfo tdi = new ThumbDirInfo(_fileNames, _parentDir);
		queue.add(tdi);

		// resume thread
		threadSuspended = !threadSuspended;
                if (!threadSuspended)
                   notify();
                else
                   threadSuspended = false;

	}

	private void createThumbs(Vector fileNames, File parentDir) {

		String tNailName;
		int scaleWidth = 140;

                String parent = parentDir.getAbsolutePath() + ImageViewer.slashS;
                String inDirS = parentDir.getName();

	        if ( inDirS.equals( ImagePreview.THUMBNAIL_DIR ) ) {
	           /*letRun();*/
	           return;
	        }

		String dir = parent + ImagePreview.THUMBNAIL_DIR + ImageViewer.slashS;

		int len = fileNames.size();
		for (int i = 0; i < len; i++) {

		    String filename = (String)fileNames.get(i);

		    File file = new File(parent+filename); // the file to convert to a thumbnail

		    String tName = dir + filename; // get the full name of the file
                    File tFile = new File(tName);

                    // if the thumbnail doesn't exist and creation of the thumbnail
                    //  has not previously failed, create it if the
                    //   file is not very small or not a GIF, JPEG, or PNG,
                    if (
                            ( !tFile.exists() )
                        ) {
                       long fileSize = file.length(); // get size of file
                       if ( (fileSize > ImagePreview.MAX_FILE_SIZE) || Utils.requiresJAI(file) )  {
		          try {
		               tNailName = scale.createThumbnail(file, tFile, scaleWidth);
	                  }
	                  catch (Exception e) {
	                  }
	               }
                    }
                    try {
                         sleep(10); // sleep for 50 ms to give GUI thread more CPU cycles
                    }
                    catch (InterruptedException ie) {}

		} // end for loop

	}

	/*private synchronized void letRun() {

                canRun = true;
                notifyAll(); // allow another ThumbnailThread to run

	} */

}

class ThumbDirInfo {

	private Vector fileNames;
	private File parentDir;

	public ThumbDirInfo(Vector fileNames, File parentDir) {

		this.fileNames = fileNames;
		this.parentDir = parentDir;

	}

	public Vector getFileNames() {
		return fileNames;
	}

	public File getParentDir() {
		return parentDir;
	}

}
