import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Right-click "View" menu for internal frames and grab dragging.
 */
class ViewPopupMenu extends JPopupMenu
                    implements MouseInputListener,
                               KeyListener {

	private ImageViewer parent;
	private ViewMenu viewMenu;

	private int m_XDifference, m_YDifference;
        private JViewport view;

        private FullScreenWindow fsWindow;
	private ImageInternalFrame selectedFrame;
	//private Container contentPane;

	private boolean fullScreen;

	public ViewPopupMenu(ImageViewer _parent) {

		super();
		parent = _parent;
		viewMenu = parent.getViewMenu();
        	createMenu();

	}

	private void createMenuItem(String text, int mnemonic, KeyStroke accelerator,
                                     ActionListener listener) {

        	JMenuItem menuItem = new JMenuItem(text);
                menuItem.setMnemonic(mnemonic);
                menuItem.setAccelerator(accelerator);
                menuItem.addActionListener(listener);
                add(menuItem);

        }

        private void createMenu() {

		createMenuItem("Close", KeyEvent.VK_C,
                                KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(parent, "closeAction") );
                addSeparator();
                createMenuItem("Save", KeyEvent.VK_S,
                                KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(parent, "saveAction") );
                addSeparator();
        	createMenuItem("Scale...", KeyEvent.VK_E,
                                KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "scaleAction") );
                createMenuItem("Zoom In", KeyEvent.VK_I,
                                KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoomInAction") );
                createMenuItem("Zoom Out", KeyEvent.VK_O,
                                KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoomOutAction") );
                createMenuItem("Zoom 1:1", KeyEvent.VK_1,
                                KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoom1To1Action") );
                addSeparator();
                createMenuItem("Minimize", KeyEvent.VK_M,
                                KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "minAction") );
                createMenuItem("Maximize", KeyEvent.VK_X,
                                KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "maxAction") );
                addSeparator();
                createMenuItem("Full Screen", KeyEvent.VK_U,
                                KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "fullScreenAction") );

        }

        public void keyTyped(KeyEvent e) {

        	System.out.println("Key Pressed");
                int keyCode =  e.getKeyCode();
                if (keyCode == KeyEvent.VK_ESCAPE) {
                   System.out.println("ESC Pressed");
                   closeFullScreenWindow();
                }

        }

        public void keyPressed(KeyEvent e) {

        	System.out.println("Key Pressed");
                int keyCode =  e.getKeyCode();
                if (keyCode == KeyEvent.VK_ESCAPE) {
                   System.out.println("ESC Pressed");
                   closeFullScreenWindow();
                }

        }

        public void keyReleased(KeyEvent e) {

        	System.out.println("Key Pressed");
                int keyCode =  e.getKeyCode();
                if (keyCode == KeyEvent.VK_ESCAPE) {
                   System.out.println("ESC Pressed");
                   closeFullScreenWindow();
                }

        }


        // empty mouse methods must be here so that this
        //  class meets the mouse input listener interface

        public void mouseMoved(MouseEvent e) {}

        public void mouseClicked(MouseEvent me) {

        	if (fullScreen)
                   closeFullScreenWindow();

        }

        public void mouseExited(MouseEvent me) {
              if (!fullScreen) {
        	Component c = (Component)me.getSource();

        	c.setCursor( Cursor.getPredefinedCursor(
                              Cursor.DEFAULT_CURSOR) );
              }
        }

        public void mouseEntered(MouseEvent me) {
              if (!fullScreen) {
        	Component c = (Component)me.getSource();

        	// use "drag" cursor
        	c.setCursor( Cursor.getPredefinedCursor(
                              Cursor.MOVE_CURSOR) );
              }
        }

        public void mousePressed(MouseEvent me) {

		if (me.isPopupTrigger())
                   show(me.getComponent(), me.getX(), me.getY());

                else if (!fullScreen) {
                   // grab dragging code
                   m_XDifference = me.getX();
                   m_YDifference = me.getY();
                }

	}

        /**
         * Invoked when a mouse button is released - used
         * to listen for mouse clicks on the images,
         * so that the resize window can be opened.
         */
        public void mouseReleased(MouseEvent me) {

		if (me.isPopupTrigger())
                   show(me.getComponent(), me.getX(), me.getY());

        }

        public void mouseDragged(MouseEvent e) {
            if (!fullScreen) {
               JScrollPane sp = (JScrollPane)e.getSource();
               view = sp.getViewport();
               DisplayJAI djai = (DisplayJAI)view.getView();

               Point p = view.getViewPosition();
               int newX = p.x - ( e.getX() - m_XDifference );
               int newY = p.y - ( e.getY() - m_YDifference );

	       RenderedImage image = djai.getImage();

               int maxX = image.getWidth() - view.getWidth();
               int maxY = image.getHeight() - view.getHeight();
               if (newX < 0)
                  newX = 0;
               if (newX > maxX)
                  newX = maxX;
               if (newY < 0)
                  newY = 0;
               if (newY > maxY)
                  newY = maxY;

               view.setViewPosition( new Point(newX, newY) );
            }
        }

        public void enableFullScreenMode(FullScreenWindow fsw, ImageInternalFrame sf) {

		fullScreen = true;

		fsWindow = fsw;
		selectedFrame = sf;
		//contentPane = selectedFrame.getContentPane();


        }

        private void closeFullScreenWindow() {

        	if (fsWindow != null) {
        	   fullScreen = false;
        	   //( (JComponent)contentPane ).setToolTipText("");
                   fsWindow.setVisible(false);
                   //selectedFrame.setContentPane(contentPane);
                   //selectedFrame.revalidate();
                   fsWindow.dispose();
                }

        }

}
